@@ -569,10 +569,14 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
         case 'I':
           {
             Priority=atoi(Switch+2);
+            if (Priority<0 || Priority>15)
+              BadSwitch(Switch);
             const char *ChPtr=strchr(Switch+2,':');
             if (ChPtr!=NULL)
             {
               SleepTime=atoi(ChPtr+1);
+              if (SleepTime>1000)
+                BadSwitch(Switch);
               InitSystemOptions(SleepTime);
             }
             SetPriority(Priority);
@@ -676,12 +680,17 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
     case 'Z':
       if (Switch[1]==0 && (!WidePresent || SwitchW[1]==0))
       {
+#ifndef GUI // stdin is not supported by WinRAR.
         // If comment file is not specified, we read data from stdin.
         strcpy(CommentFile,"stdin");
+#endif
+      }
+      else
+      {
+        strncpyz(CommentFile,Switch+1,ASIZE(CommentFile));
+        if (WidePresent)
+          wcsncpyz(CommentFileW,SwitchW+1,ASIZE(CommentFileW));
       }
-      strncpyz(CommentFile,Switch+1,ASIZE(CommentFile));
-      if (WidePresent)
-        wcsncpyz(CommentFileW,SwitchW+1,ASIZE(CommentFileW));
       break;
     case 'M':
       switch(etoupper(Switch[1]))
@@ -690,7 +699,7 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
           {
             const char *Str=Switch+2;
             if (*Str=='-')
-              for (int I=0;I<sizeof(FilterModes)/sizeof(FilterModes[0]);I++)
+              for (uint I=0;I<ASIZE(FilterModes);I++)
                 FilterModes[I].State=FILTER_DISABLE;
             else
               while (*Str)
@@ -764,10 +773,10 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
             }
           }
           break;
-#ifdef PACK_SMP
+#ifdef RAR_SMP
         case 'T':
           Threads=atoi(Switch+2);
-          if (Threads>16)
+          if (Threads>MaxPoolThreads || Threads<1)
             BadSwitch(Switch);
           else
           {
@@ -798,62 +807,7 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
           VolSize=0;
           break;
         default:
-          {
-            int64 NewVolSize=atoil(&Switch[1]);
-
-            if (NewVolSize==0)
-              NewVolSize=INT64NDF; // Autodetecting volume size.
-            else
-              switch (Switch[strlen(Switch)-1])
-              {
-                case 'f':
-                case 'F':
-                  switch(NewVolSize)
-                  {
-                    case 360:
-                      NewVolSize=362496;
-                      break;
-                    case 720:
-                      NewVolSize=730112;
-                      break;
-                    case 1200:
-                      NewVolSize=1213952;
-                      break;
-                    case 1440:
-                      NewVolSize=1457664;
-                      break;
-                    case 2880:
-                      NewVolSize=2915328;
-                      break;
-                  }
-                  break;
-                case 'k':
-                  NewVolSize*=1024;
-                  break;
-                case 'm':
-                  NewVolSize*=1024*1024;
-                  break;
-                case 'M':
-                  NewVolSize*=1000*1000;
-                  break;
-                case 'g':
-                  NewVolSize*=1024*1024*1024;
-                  break;
-                case 'G':
-                  NewVolSize*=1000*1000*1000;
-                  break;
-                case 'b':
-                case 'B':
-                  break;
-                default:
-                  NewVolSize*=1000;
-                  break;
-              }
-            if (VolSize==0)
-              VolSize=NewVolSize;
-            else
-              NextVolSizes.Push(NewVolSize);
-          }
+          VolSize=INT64NDF; // UnRAR -v switch for list command.
           break;
       }
       break;
@@ -978,7 +932,7 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
       break;
 #ifndef GUI
     case '?' :
-      OutHelp();
+      OutHelp(RARX_SUCCESS);
       break;
 #endif
     default :
@@ -993,7 +947,7 @@ void CommandData::ProcessSwitch(const char *Switch,const wchar *SwitchW)
 void CommandData::BadSwitch(const char *Switch)
 {
   mprintf(St(MUnknownOption),Switch);
-  ErrHandler.Exit(USER_ERROR);
+  ErrHandler.Exit(RARX_USERERROR);
 }
 #endif
 
@@ -1039,7 +993,7 @@ inline bool CmpMSGID(MSGID i1,MSGID i2)
 #endif
 }
 
-void CommandData::OutHelp()
+void CommandData::OutHelp(RAR_EXIT ExitCode)
 {
 #if !defined(GUI) && !defined(SILENT)
   OutTitle();
@@ -1097,7 +1051,7 @@ void CommandData::OutHelp()
     if (CmpMSGID(Help[I],MCHelpSwOL))
       continue;
 #endif
-#ifndef PACK_SMP
+#ifndef RAR_SMP
     if (CmpMSGID(Help[I],MCHelpSwMT))
       continue;
 #endif
@@ -1116,7 +1070,7 @@ void CommandData::OutHelp()
     mprintf(St(Help[I]));
   }
   mprintf("\n");
-  ErrHandler.Exit(USER_ERROR);
+  ErrHandler.Exit(ExitCode);
 #endif
 }
 
@@ -1183,7 +1137,7 @@ bool CommandData::ExclCheckArgs(StringList *Args,bool Dir,char *CheckName,bool C
     if (CheckFullPath && IsFullPath(CurMask))
     {
       // We do not need to do the special "*\" processing here, because
-      // onlike the "else" part of this "if", now we convert names to full
+      // unlike the "else" part of this "if", now we convert names to full
       // format, so they all include the path, which is matched by "*\"
       // correctly. Moreover, removing "*\" from mask would break
       // the comparison, because now all names have the path.
@@ -1332,7 +1286,7 @@ void CommandData::ProcessCommand()
 
   const char *SingleCharCommands="FUADPXETK";
   if (Command[0]!=0 && Command[1]!=0 && strchr(SingleCharCommands,*Command)!=NULL || *ArcName==0)
-    OutHelp();
+    OutHelp(*Command==0 ? RARX_SUCCESS:RARX_USERERROR); // Return 'success' for 'rar' without parameters.
 
 #ifdef _UNIX
   if (GetExt(ArcName)==NULL && (!FileExist(ArcName) || IsDir(GetFileAttr(ArcName))))
@@ -1376,7 +1330,7 @@ void CommandData::ProcessCommand()
       ListArchive(this);
       break;
     default:
-      OutHelp();
+      OutHelp(RARX_USERERROR);
 #endif
   }
   if (!BareOutput)
