@@ -6,7 +6,7 @@
 #define   MRetryAbort        "_Retry_Abort"
 #define   MCopyright         "\nRAR %s   Copyright (c) 1993-%d Alexander Roshal   %d %s %d"
 #define   MRegTo             "\nRegistered to %s\n"
-#define   MShare             "\nShareware version         Type RAR -? for help\n"
+#define   MShare             "\nTrial version             Type RAR -? for help\n"
 #define   MUCopyright        "\nUNRAR %s freeware      Copyright (c) 1993-%d Alexander Roshal\n"
 #define   MBeta              "beta"
 #define   MMonthJan          "Jan"
@@ -50,6 +50,7 @@
 #define   MCHelpCmdX         "\n  x             Extract files with full path"
 #define   MCHelpSw           "\n\n<Switches>"
 #define   MCHelpSwm          "\n  -             Stop switches scanning"
+#define   MCHelpSwAT         "\n  @[+]          Disable [enable] file lists"
 #define   MCHelpSwAC         "\n  ac            Clear Archive attribute after compression or extraction"
 #define   MCHelpSwAD         "\n  ad            Append archive name to destination path"
 #define   MCHelpSwAG         "\n  ag[format]    Generate archive name using the current date"
@@ -87,6 +88,7 @@
 #define   MCHelpSwISND       "\n  isnd          Enable sound"
 #define   MCHelpSwK          "\n  k             Lock archive"
 #define   MCHelpSwKB         "\n  kb            Keep broken extracted files"
+#define   MCHelpSwLog        "\n  log[f][=name] Write names to log file"
 #define   MCHelpSwMn         "\n  m<0..5>       Set compression level (0-store...3-default...5-maximal)"
 #define   MCHelpSwMC         "\n  mc<par>       Set advanced compression parameters"
 #define   MCHelpSwMD         "\n  md<size>      Dictionary size in KB (64,128,256,512,1024,2048,4096 or A-G)"
@@ -298,7 +300,7 @@
 #define   MErrCreateLnk      "\nWARNING: Cannot create link %s"
 #define   MSymLinkExists     "\nWARNING: Symbolic link %s already exists"
 #define   MAskRetryCreate    "\nCannot create %s. Retry ?"
-#define   MListMACHead1      "\n MacOS file type:  %c%c%c%c  ; "
+#define   MListMACHead1      "\n Mac OS file type:  %c%c%c%c  ; "
 #define   MListMACHead2      "file creator:  %c%c%c%c\n"
 #define   MDataBadCRC        "\n%-20s : packed data CRC failed in volume %s"
 #define   MFileRO            "\n%s is read-only"
