@@ -6,22 +6,29 @@
 #define DEFAULT_RECVOLUMES  -10
 
 enum PATH_EXCL_MODE {
-  EXCL_UNCHANGED,EXCL_BASEPATH,EXCL_SKIPWHOLEPATH,EXCL_SAVEFULLPATH,
-  EXCL_SKIPABSPATH,EXCL_ABSPATH
+  EXCL_UNCHANGED=0,    // Process paths as is (default).
+  EXCL_SKIPWHOLEPATH,  // -ep  (exclude the path completely)
+  EXCL_BASEPATH,       // -ep1 (exclude the base part of path)
+  EXCL_SAVEFULLPATH,   // -ep2 (the full path without the disk letter)
+  EXCL_ABSPATH,        // -ep3 (the full path with the disk letter)
+
+  EXCL_SKIPABSPATH     // Works as EXCL_BASEPATH for fully qualified paths
+                       // and as EXCL_UNCHANGED for relative paths.
+                       // Used by WinRAR GUI only.
 };
 
 enum {SOLID_NONE=0,SOLID_NORMAL=1,SOLID_COUNT=2,SOLID_FILEEXT=4,
       SOLID_VOLUME_DEPENDENT=8,SOLID_VOLUME_INDEPENDENT=16};
 
-enum {ARCTIME_NONE,ARCTIME_KEEP,ARCTIME_LATEST};
+enum {ARCTIME_NONE=0,ARCTIME_KEEP,ARCTIME_LATEST};
 
 enum EXTTIME_MODE {
-  EXTTIME_NONE,EXTTIME_1S,EXTTIME_HIGH1,EXTTIME_HIGH2,EXTTIME_HIGH3
+  EXTTIME_NONE=0,EXTTIME_1S,EXTTIME_HIGH1,EXTTIME_HIGH2,EXTTIME_HIGH3
 };
 
-enum {NAMES_ORIGINALCASE,NAMES_UPPERCASE,NAMES_LOWERCASE};
+enum {NAMES_ORIGINALCASE=0,NAMES_UPPERCASE,NAMES_LOWERCASE};
 
-enum MESSAGE_TYPE {MSG_STDOUT,MSG_STDERR,MSG_ERRONLY,MSG_NULL};
+enum MESSAGE_TYPE {MSG_STDOUT=0,MSG_STDERR,MSG_ERRONLY,MSG_NULL};
 
 enum RECURSE_MODE 
 {
@@ -42,7 +49,7 @@ enum OVERWRITE_MODE
 
 enum RAR_CHARSET { RCH_DEFAULT=0,RCH_ANSI,RCH_OEM,RCH_UNICODE };
 
-#define     MAX_FILTERS           16
+#define     MAX_FILTER_TYPES           16
 enum FilterState {FILTER_DEFAULT=0,FILTER_AUTO,FILTER_FORCE,FILTER_DISABLE};
 
 
@@ -53,6 +60,8 @@ struct FilterMode
   int Param2;
 };
 
+#define MAX_GENERATE_MASK  128
+
 
 class RAROptions
 {
@@ -66,6 +75,7 @@ class RAROptions
     bool InclAttrSet;
     uint WinSize;
     char TempPath[NM];
+    bool ConfigDisabled; // Switch -cfg-.
     char ExtrPath[NM];
     wchar ExtrPathW[NM];
     char CommentFile[NM];
@@ -111,6 +121,10 @@ class RAROptions
     bool KeepBroken;
     bool OpenShared;
     bool DeleteFiles;
+#ifndef SFX_MODULE
+    bool GenerateArcName;
+    char GenerateMask[MAX_GENERATE_MASK];
+#endif
     bool SyncFiles;
     bool ProcessEA;
     bool SaveStreams;
@@ -124,7 +138,7 @@ class RAROptions
     bool Lock;
     bool Test;
     bool VolumePause;
-    FilterMode FilterModes[MAX_FILTERS];
+    FilterMode FilterModes[MAX_FILTER_TYPES];
     char EmailTo[NM];
     uint VersionControl;
     bool NoEndBlock;
@@ -136,7 +150,7 @@ class RAROptions
     EXTTIME_MODE xarctime;
     char CompressStdin[NM];
 
-#ifdef PACK_SMP
+#ifdef RAR_SMP
     uint Threads;
 #endif
 
