@@ -4,22 +4,23 @@
 #include "raros.hpp"
 #include "os.hpp"
 
-
 #ifdef RARDLL
 #include "dll.hpp"
 #endif
 
+
+#ifndef _WIN_CE
 #include "version.hpp"
+#endif
 #include "rartypes.hpp"
 #include "rardefs.hpp"
 #include "rarlang.hpp"
-#include "int64.hpp"
 #include "unicode.hpp"
 #include "errhnd.hpp"
 #include "array.hpp"
 #include "timefn.hpp"
+#include "options.hpp"
 #include "headers.hpp"
-#include "rarfn.hpp"
 #include "pathfn.hpp"
 #include "strfn.hpp"
 #include "strlist.hpp"
@@ -35,34 +36,34 @@
 #include "savepos.hpp"
 #include "getbits.hpp"
 #include "rdwrfn.hpp"
-#include "options.hpp"
 #include "archive.hpp"
 #include "match.hpp"
 #include "cmddata.hpp"
 #include "filcreat.hpp"
 #include "consio.hpp"
 #include "system.hpp"
+#ifdef _WIN_ALL
 #include "isnt.hpp"
+#endif
 #include "log.hpp"
 #include "rawread.hpp"
 #include "encname.hpp"
 #include "resource.hpp"
 #include "compress.hpp"
 
-
 #include "rarvm.hpp"
 #include "model.hpp"
 
-
 #include "unpack.hpp"
 
 
+
 #include "extinfo.hpp"
 #include "extract.hpp"
 
 
-#include "list.hpp"
 
+#include "list.hpp"
 
 
 #include "rs.hpp"
